/*******************************************************
**  
** Helps manipulates registry.
** 
** 11 Mar, 2000
** Mark Stroetzel Glasberg
**
********************************************************/

#include <stdio.h>
#include <windows.h>
#include <winreg.h>
#include "registry.h"


// Returns Window definitions of keynames
static HKEY gethkey(const char *keyname)
{
  if(!strcmp(keyname, "HKEY_CLASSES_ROOT"))
    return HKEY_CLASSES_ROOT;
  else if(!strcmp(keyname, "HKEY_CURRENT_USER"))
    return HKEY_CURRENT_USER;
  else if(!strcmp(keyname, "HKEY_LOCAL_MACHINE"))
    return HKEY_LOCAL_MACHINE;
  else if(!strcmp(keyname, "HKEY_USERS"))
    return HKEY_USERS;
  else if(!strcmp(keyname, "HKEY_CURRENT_CONFIG"))
    return HKEY_CURRENT_CONFIG;
  return NULL;
}


// Reads a value at the registry. Max size is define at registry.h
// If value does not exist, returns NULL;
// hkey     -> ex. HKEY_CURRENT_USER
// regpath  -> ex. "Software\\Netscape\\Netscape Navigator\\biff"
// regvalue -> ex. "CurrentUser"
// Retuns an already allocated string (user must free it)
char *Registry::readregistry(const char *name, const char *regpath, const char *regvalue)
{
  HKEY hkey = gethkey(name);
  HKEY  hkMain;
  LONG  test;
  char  tmpval[REG_BUFFER_SIZE];
  char  *val = (char*) calloc(REG_BUFFER_SIZE, sizeof(char));
  memset(tmpval, 0, REG_BUFFER_SIZE);
  DWORD valtype, 
        valsize = REG_BUFFER_SIZE; 

  test = RegOpenKeyEx(hkey, regpath, 0, KEY_READ, &hkMain); 
  if(test!=ERROR_SUCCESS) 
  {
    RegCloseKey (hkMain);
    return NULL;
  }

  test = RegQueryValueEx(hkMain,         // handle of key to query 
                        regvalue,        // address of name of value to query 
                        0,               // reserved 
                        &valtype,        // address of buffer for value type 
                        (LPBYTE)&tmpval, // address of data buffer 
                        &valsize         // address of data buffer size 
                        ); 

  if(test!=ERROR_SUCCESS) /* Value does not exist or error */
  {
    RegCloseKey (hkMain);
    return NULL;
  }

  strcpy(val, tmpval);
  RegCloseKey (hkMain);

  return val;
}

// Changes a registry. If it does not exist it creates a new one.
// hkey     -> ex. HKEY_CURRENT_USER
// regpath  -> ex. "Software\\Netscape\\Netscape Navigator\\biff"
// regvalue -> ex. "CurrentUser"
// Retuns TRUE (sucess) of FALSE failure
int Registry::changeregistry(const char *name, const char *regpath, const char *regvalue, const char *newvalue)
{
  HKEY hkey = gethkey(name);
  HKEY  hkMain ;
  LONG  test;
  char  *val = (char*) malloc(sizeof(char)*REG_BUFFER_SIZE);
  DWORD valsize = 0;
  DWORD disposition;

  test = RegCreateKeyEx(hkey, regpath, 0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hkMain, &disposition);

  if(test!=ERROR_SUCCESS)
  {
    RegCloseKey (hkMain);
    return FALSE;
  }

  if(newvalue != NULL)
    valsize = (DWORD) strlen(newvalue);

  test = RegSetValueEx( hkMain,     // handle of key to set value for 
                        regvalue,   //address of value to set 
                        0,          // reserved 
                        REG_SZ,     // flag for value type 
                        (const BYTE*) newvalue,   // address of value data 
                        valsize     // size of value data 
                      ); 
 
  if(test!=ERROR_SUCCESS) 
  {
    RegCloseKey (hkMain);
    return FALSE;
  }

  RegCloseKey (hkMain);

  return TRUE;
}


// Deletes a registry. 
// hkey     -> ex. HKEY_CURRENT_USER
// regpath  -> ex. "Software\\Netscape\\Netscape Navigator\\biff"
// regvalue -> ex. "SomeData"
// Returns TRUE if the registry was deleted or returns FALSE if 
// it did not exist or in case of an error.
int Registry::deleteregistry(const char*name, const char *regpath, const char *regvalue)
{
  HKEY hkey = gethkey(name);
  HKEY  hkMain ;
  LONG  test;

  test = RegOpenKeyEx (hkey, regpath, 0, KEY_WRITE, &hkMain); 
  if(test!=ERROR_SUCCESS)
  {
    RegCloseKey (hkMain);
    return FALSE;
  }

  test = RegDeleteValue(hkMain, regvalue); 
 
  if(test!=ERROR_SUCCESS) 
  {
    RegCloseKey (hkMain);
    return FALSE;
  }

  RegCloseKey (hkMain);

  return TRUE;
}
